function z = gauss2rsum(sigma_x, sigma_y, m_x, m_y, theta, n, precision)
%GAUSS2RSUM Sum of 2-D rotated gaussians translated along the rotated
%       x-axis of the gaussians.
%       GAUSS2(SIGMA_X, SIGMA_Y, THETA, N, PRECISION) 
%       is a matrix of N rotated, 
%       discretized 2-D gaussians with standard deviation SIGMA_X in 
%       x-direction, SIGMA_Y in y-direction. The gaussians are offset by 
%       2*SIGMA_X in the direction of the rotated x-axis. The rotation of 
%       the x-axis is specified by THETA in rad. The Gaussians are
%       discretized in the interval [-PRECISION*SIGMA; PRECISION*SIGMA]
%       for both X- and Y-AXIS. IF PRECISION is omitted, it is set to the
%       default value of 3. 
%
%       Matrix size (M, N) is determined by SIGMA_X and SIGMA_Y.


if nargin < 7
  precision = 3;
end

offset = 2*sigma_x;   % offset between gaussians; 
                      % value choosen in order to generate a plateau-like 
                      % profile that smoothly rolls off at either end  

bound_x = precision * sigma_x + abs(m_x) + (n-1)/2*abs(offset);
bound_y = precision * sigma_y + abs(m_y);

bound_x_rot = round(abs(cos(theta)*bound_x) + abs(sin(theta)*bound_y));
bound_y_rot = round(abs(sin(theta)*bound_x) + abs(cos(theta)*bound_y));

x = (-bound_x_rot : bound_x_rot);
y = fliplr(-bound_y_rot : bound_y_rot);

[X, Y] = meshgrid(x, y);

z = zeros(size(y, 2), size(x, 2));
maxoffset = (n-1)/2*offset;

for m_x_offset = -maxoffset : offset : maxoffset
  z_mx = 1/(2*pi * sigma_x * sigma_y) * exp(-0.5 * ...
           ((X .* cos(theta) + Y .* sin(theta) - (m_x + m_x_offset)).^2 ...
                                                               / sigma_x^2 ...
                      + (- X .* sin(theta) + Y .* cos(theta) - m_y).^2 ...
                                                               / sigma_y^2));
% same formula, other evaluation -> no numerical effect
%  z_mx = 1/(2*pi*sigma_x*sigma_y) * exp(-0.5 * ...
%           ( ((X .* cos(theta) + Y .* sin(theta) - (m_x + m_x_offset)) ...
%                                                            / sigma_x).^2 ...
%              + ((- X .* sin(theta) + Y .* cos(theta) - m_y) / sigma_y).^2) );
  
  z = z + z_mx;             % intnorm(z_mx) unnecessary, normalized afterwards
end

z = intnorm(z);

