function W = longrange(V, scale)
%LONGRANGE  Long-range interaction.
%   Formulation for recurrent model.           
%   W = LONGRANGE(V) computes the output of the recurrent long-range
%   interaction.  
%   W = LONGRANGE(V) computes the output of the recurrent long-range
%   interaction for scale SCALE, default is 4. 
%
%% Thorsten Hansen 2008-04-08
  
if nargin < 2
  scale = 4;
end
switch scale
 case 1.5 %
  radius_dir = 9;
  sigma_sur = 3;
 case 2
  radius_dir = 13;
  sigma_sur = 4;
 case 3
  radius_dir = 19;
  sigma_sur = 6;
 case 4 % default
  radius_dir = 25;
  sigma_sur = 8; % size of Gaussian for surround inhibition
 otherwise
  error(['Invalid scale ' num2str(scale) '.'])
end

[M N Omax] = size(V);
switch Omax
case 4
  sigma_o = 0.5; % standard deviation of orientational gaussian
                 % approx. Omax/2 /3 
                 % to give zero values for orthogonal orientation
		 %
                 %   = realmax: equal weighting of all orientations
		 %   = eps: delta impulse for current orientation
		 %   = 0.5: positive weighting for direct neighbors 
  eta_minus = 2; % approx. 1/max(gausso(sigma_o, i, Omax))
case 8
  sigma_o = 1.5;  
  eta_minus = 4;
case 16
  sigma_o = 2.5;
  eta_minus = 7;
otherwise
  error('Omax not supported')
end


% directional weighting in the spatial domain
for i=1:Omax
  theta = (i-1)*pi/Omax;
  i_orthogonal = mod(i-1+Omax/2, Omax)+1; % orientation orthogonal to 
                                      %   current orientation i
  % excitatory directional input
  B = bipole(theta, rad(10), 1, radius_dir, 3);
  net_theta_plus(:,:,i) = correlate2(hwrect(V(:,:,i) - V(:,:,i_orthogonal)), B);

  % inhibitory directional/isotropic input  
  net_theta_spatial(:,:,i) = conv2gauss(net_theta_plus(:,:,i), sigma_sur);
end

alpha_W = 0.2;
eta_plus = 5;
beta_W = 1/1000;

for i=1:Omax
  % inhibitory surround input: orientational weighting
  g_o = gausso(sigma_o, i, Omax);
  net_theta_minus = zeros(M, N);
  for o=1:Omax
    net_theta_minus = net_theta_minus + net_theta_spatial(:,:,o)*g_o(o);
  end;
  
  % long-range interaction (Eq. 14)
  W(:,:,i) = beta_W * V(:,:,i) .* (1 + eta_plus*net_theta_plus(:,:,i)) ...
                               ./ (alpha_W + eta_minus*net_theta_minus);
end

