function [C Wall] = modellr(I, runs, Omax, scale)
%MODELLR  A recurrent model of contour integration in primary visual cortex.
%
%   [C WALL] = MODELLR(I, RUNS) runs the model on input stimulus I with a
%   number of RUNS recurrent interactions. MODELLR returns the output
%   of the initial complex cell responses C and the long-range stage
%   WALL of all recurrent interactions. 
% 
%   [C Wall] = MODELLR(I, RUNS, OMAX) sets the number of orientations,
%   default is 4.
%    
%   [C Wall] = MODELLR(I, RUNS, OMAX, SCALE) sets the number of orientations,
%   and the scale, default is 4.
%
% Thorsten Hansen 2008-04-08
  
if nargin < 2
  error('usage: modellr(I, runs)')
end

if nargin < 3
  Omax = 4; % number of orientations
end

if nargin < 4
  scale = 4; % scale of long-range interaction, see function longrange
end


%----------------------------------------
% check range of input image
%----------------------------------------

if gmax(I) > 1 || gmin(I) < 0 
  error('Input image must be in the range [0, 1].')
end

%----------------------------------------
% LGN on and off cells
%----------------------------------------
sigma_c         = 1.00;
sigma_s         = 3.00;  % 6.00
K = correlate2(I, dog2(sigma_c, sigma_s));
K_on = hwrect(K);
K_off = hwrect(-K);


%----------------------------------------
% simple cells
%----------------------------------------
strdisp(['**** Omax: ' int2str(Omax) ' ***'])
[S_ld S_dl] = simplecells(K_on, K_off, Omax);
strdisp(['simple ld: min ' num2str(gmin(S_ld))]); 
strdisp(['         : max ' num2str(gmax(S_ld))]); 
strdisp(['       dl: min ' num2str(gmin(S_dl))]); 
strdisp(['         : max ' num2str(gmax(S_dl))]); 


%----------------------------------------
% complex cells
%----------------------------------------
C = S_ld + S_dl;

strdisp(['complex min ' num2str(gmin(C))]);
strdisp(['        max ' num2str(gmax(C))]);

if runs == 0
  Wall = 0; % just to define a dummy return value
end


%
% feedback loop
%
for i=1:runs

  fprintf('.') % progress dot

  %----------------------------------------
  % combination stage
  %----------------------------------------
  alpha_V = 0.2;
  beta_V = 10;
  delta_V = 2;

  if i == 1 % first recurrent loop
    W = C; 
  end

  net_theta = C + delta_V*W; % Eq. 6
  
  V = beta_V * net_theta ./ (alpha_V + net_theta); % Eq. 7
  
  
  strdisp(['net_theta_CV6 min ' num2str(gmin(net_theta))]);
  strdisp(['            max ' num2str(gmax(net_theta))]);
  strdisp(['V1(t = ' int2str(i) ') min ' num2str(gmin(V))]);
  strdisp(['          max ' num2str(gmax(V))]);


  %----------------------------------------
  % long-range interaction
  %----------------------------------------
  W = longrange(V, scale);

  strdisp(['W(t = ' int2str(i) ') min ' num2str(gmin(W))]);
  strdisp(['          max ' num2str(gmax(W))]);

  Wall(:,:,:,i) = W;
end
fprintf('\n')



